@@ -3,7 +3,7 @@
 #ifndef R2_ANAL_H
 #define R2_ANAL_H
 
-#define USE_DICT 0
+#define USE_DICT 1
 
 /* use sdb function storage */
 #define FCN_SDB 1
@@ -252,7 +252,7 @@ struct r_anal_attr_t {
 };
 
 typedef struct r_anal_fcn_store_t {
-	SdbHash *h;
+	SdbHt *h;
 	RList *l;
 } RAnalFcnStore;
 
@@ -296,10 +296,6 @@ typedef struct r_anal_type_function_t {
 	RList *fcn_locs; //sorted list of a function *.loc refs
 	//RList *locals; // list of local labels -> moved to anal->sdb_fcns
 	RList *bbs;
-#if FCN_OLD
-	RList *refs;
-	RList *xrefs;
-#endif
 	RAnalFcnMeta meta;
 	RRangeTiny bbr;
 	RBNode rb;
@@ -339,6 +335,7 @@ enum {
 	R_META_TYPE_COMMENT = 'C',
 	R_META_TYPE_RUN = 'r',
 	R_META_TYPE_HIGHLIGHT = 'H',
+	R_META_TYPE_VARTYPE = 't',
 };
 
 // anal
@@ -350,6 +347,7 @@ enum {
 	R_ANAL_OP_FAMILY_SSE,    /* extended multimedia instruction (packed data) */
 	R_ANAL_OP_FAMILY_PRIV,   /* priviledged instruction */
 	R_ANAL_OP_FAMILY_CRYPTO, /* cryptographic instructions */
+	R_ANAL_OP_FAMILY_THREAD, /* thread/lock/sync instructions */
 	R_ANAL_OP_FAMILY_VIRT,   /* virtualization instructions */
 	R_ANAL_OP_FAMILY_IO,     /* IO instructions (i.e. IN/OUT) */
 	R_ANAL_OP_FAMILY_LAST
@@ -402,7 +400,9 @@ typedef enum {
 	R_ANAL_OP_TYPE_IJMP  = R_ANAL_OP_TYPE_IND | R_ANAL_OP_TYPE_UJMP,
 	R_ANAL_OP_TYPE_IRJMP = R_ANAL_OP_TYPE_IND | R_ANAL_OP_TYPE_REG | R_ANAL_OP_TYPE_UJMP,
 	R_ANAL_OP_TYPE_CJMP  = R_ANAL_OP_TYPE_COND | R_ANAL_OP_TYPE_JMP,  /* conditional jump */
-	R_ANAL_OP_TYPE_MJMP  = R_ANAL_OP_TYPE_MEM | R_ANAL_OP_TYPE_JMP,  /* conditional jump */
+	R_ANAL_OP_TYPE_RCJMP = R_ANAL_OP_TYPE_REG | R_ANAL_OP_TYPE_CJMP,  /* conditional jump register */
+	R_ANAL_OP_TYPE_MJMP  = R_ANAL_OP_TYPE_MEM | R_ANAL_OP_TYPE_JMP,   /* memory jump */
+	R_ANAL_OP_TYPE_MCJMP = R_ANAL_OP_TYPE_MEM | R_ANAL_OP_TYPE_CJMP,  /* memory conditional jump */
 	R_ANAL_OP_TYPE_UCJMP = R_ANAL_OP_TYPE_COND | R_ANAL_OP_TYPE_UJMP, /* conditional unknown jump */
 	R_ANAL_OP_TYPE_CALL  = 3,  /* call to subroutine (branch+link) */
 	R_ANAL_OP_TYPE_UCALL = 4, /* unknown call (register or so) */
@@ -464,8 +464,10 @@ typedef enum {
 } _RAnalOpType;
 
 typedef enum {
-	R_ANAL_OP_MASK_ESIL       = 1,
-	R_ANAL_OP_MASK_ALL        = R_ANAL_OP_MASK_ESIL
+	R_ANAL_OP_MASK_BASIC      = 0, // Just fills basic op info , it's fast
+	R_ANAL_OP_MASK_ESIL       = 1, // It fills RAnalop->esil info
+	R_ANAL_OP_MASK_VAL        = 2, // It fills RAnalop->dst/src info
+	R_ANAL_OP_MASK_ALL        = R_ANAL_OP_MASK_ESIL | R_ANAL_OP_MASK_VAL
 } RAnalOpMask;
 
 /* TODO: what to do with signed/unsigned conditionals? */
@@ -574,24 +576,24 @@ typedef struct r_anal_switch_obj_t {
 	RList *cases;
 } RAnalSwitchOp;
 
-#define RANAL void*
-//struct r_anal_t*
-#define RANAL_BLOCK void*
-//struct r_anal_bb_t*
+struct r_anal_t;
+struct r_anal_bb_t;
 typedef struct r_anal_callbacks_t {
-	int (*on_fcn_new) (RANAL, void *user, RAnalFunction *fcn);
-	int (*on_fcn_delete) (RANAL , void *user, RAnalFunction *fcn);
-	int (*on_fcn_rename) (RANAL, void *user, RAnalFunction *fcn, const char *oldname);
-	int (*on_fcn_bb_new) (RANAL, void *user, RAnalFunction *fcn, RANAL_BLOCK bb);
+	int (*on_fcn_new) (struct r_anal_t *, void *user, RAnalFunction *fcn);
+	int (*on_fcn_delete) (struct r_anal_t *, void *user, RAnalFunction *fcn);
+	int (*on_fcn_rename) (struct r_anal_t *, void *user, RAnalFunction *fcn, const char *oldname);
+	int (*on_fcn_bb_new) (struct r_anal_t *, void *user, RAnalFunction *fcn, struct r_anal_bb_t *bb);
 } RAnalCallbacks;
 
 #define R_ANAL_ESIL_GOTO_LIMIT 4096
 
 typedef struct r_anal_options_t {
 	int depth;
+	bool vars; //analyze local var and arguments
 	int cjmpref;
 	int jmpref;
 	int jmpabove;
+	bool ijmp;
 	int followdatarefs;
 	int searchstringrefs;
 	int followbrokenfcnsrefs;
@@ -600,13 +602,13 @@ typedef struct r_anal_options_t {
 	int afterjmp; // continue analysis after jmp eax or forward jmp // option
 	int recont; // continue on recurse analysis mode
 	int eobjmp; // option
-	int bbsplit;
 	int noncode;
 	int nopskip; // skip nops at the beginning of functions
 	int hpskip; // skip `mov reg,reg` and `lea reg,[reg]`
 	int jmptbl; // analyze jump tables
 	bool pushret; // analyze push+ret as jmp
 	bool armthumb; //
+	bool endsize; // chop function size which is known to be buggy but goodie too
 } RAnalOptions;
 
 typedef enum {
@@ -620,7 +622,6 @@ typedef struct r_anal_t {
 	int bits;
 	int lineswidth; // wtf
 	int big_endian;
-	int split; // used only from core
 	int sleep; // sleep some usecs before analyzing more (avoid 100% cpu usages)
 	RAnalCPPABI cpp_abi;
 	void *user;
@@ -647,20 +648,19 @@ typedef struct r_anal_t {
 	int esil_goto_limit;
 	int pcalign;
 	int bitshift;
-	RList *types;
 	//struct r_anal_ctx_t *ctx;
 	struct r_anal_esil_t *esil;
 	struct r_anal_plugin_t *cur;
 	RAnalRange *limit;
 	RList *plugins;
-	Sdb *sdb_xrefs;
 	Sdb *sdb_types;
+	Sdb *sdb_fmts;
 	Sdb *sdb_meta; // TODO: Future r_meta api
 	Sdb *sdb_zigns;
 
 #if USE_DICT
-	dict *dict_refs;
-	dict *dict_xrefs;
+	SdbHt *dict_refs;
+	SdbHt *dict_xrefs;
 #endif
 	bool recursive_noreturn;
 	RSpaces meta_spaces;
@@ -669,7 +669,6 @@ typedef struct r_anal_t {
 	PrintfCallback cb_printf;
 	//moved from RAnalFcn
 	Sdb *sdb; // root
-	Sdb *sdb_refs;
 	Sdb *sdb_fcns;
 	Sdb *sdb_pins;
 #define DEPRECATE 1
@@ -690,6 +689,7 @@ typedef struct r_anal_t {
 	RList /*RAnalRange*/ *bits_ranges;
 	RListComparator columnSort;
 	int stackptr;
+	bool fillval;
 	bool (*log)(struct r_anal_t *anal, const char *msg);
 	bool (*read_at)(struct r_anal_t *anal, ut64 addr, ut8 *buf, int len);
 	char *cmdtail;
@@ -708,6 +708,7 @@ typedef struct r_anal_hint_t {
 	ut64 ptr;
 	ut64 jump;
 	ut64 fail;
+	ut64 ret; // hint for function ret values
 	char *arch;
 	char *opcode;
 	char *syntax;
@@ -739,10 +740,11 @@ typedef struct r_anal_var_access_t {
 typedef struct r_anal_var_t {
 	char *name;  /* name of the variable */
 	char *type;  // cparse type of the variable
-	char kind;   // 'a'rg, 'v'ar ..
+	char kind;   // reg , stack ...
 	ut64 addr;   // not used correctly?
 	ut64 eaddr;  // not used correctly?
 	int size;
+	bool isarg;
 	int delta;   /* delta offset inside stack frame */
 	int scope;   /* global, local... | in, out... */
 	/* probably dupped or so */
@@ -786,6 +788,7 @@ typedef struct r_anal_op_t {
 	int family;     /* family of opcode */
 	int id;         /* instruction id */
 	bool eob;       /* end of block (boolean) */
+	bool sign;      /* operates on signed values, false by default */
 	/* Run N instructions before executing the current one */
 	int delay;      /* delay N slots (mips, ..)*/
 	ut64 jump;      /* true jmp */
@@ -830,6 +833,7 @@ typedef struct r_anal_bb_t {
 	int returnbb;
 	int conditional;
 	int traced;
+	ut32 colorize;
 	char *label;
 	ut8 *fingerprint;
 	RAnalDiff *diff;
@@ -866,9 +870,9 @@ typedef enum {
 } RAnalRefType;
 
 typedef struct r_anal_ref_t {
-	int type;
 	ut64 addr;
 	ut64 at;
+	RAnalRefType type;
 } RAnalRef;
 
 /* represents a reference line from one address (from) to another (to) */
@@ -891,7 +895,7 @@ typedef struct r_anal_state_type_t {
 	ut64 current_addr;
 	ut64 next_addr;
 	RList *bbs;
-	SdbHash *ht;
+	SdbHt *ht;
 	ut64 ht_sz;
 	RAnalFunction *current_fcn;
 	RAnalOp *current_op;
@@ -938,8 +942,10 @@ enum {
 	R_ANAL_TRAP_WRITE_ERR = 4,
 	R_ANAL_TRAP_READ_ERR = 5,
 	R_ANAL_TRAP_EXEC_ERR = 6,
-	R_ANAL_TRAP_TODO = 7,
-	R_ANAL_TRAP_HALT = 8,
+	R_ANAL_TRAP_INVALID = 7,
+	R_ANAL_TRAP_UNALIGNED = 8,
+	R_ANAL_TRAP_TODO = 9,
+	R_ANAL_TRAP_HALT = 10,
 };
 
 enum {
@@ -1021,6 +1027,35 @@ typedef struct r_anal_reil {
 #define ESIL_STACK_NAME "esil.ram"
 #define ESIL struct r_anal_esil_t
 
+typedef struct r_anal_esil_source_t {
+	ut32 id;
+	ut32 claimed;
+	void *content;
+} RAnalEsilSource;
+
+R_API void r_anal_esil_sources_init(ESIL *esil);
+R_API ut32 r_anal_esil_load_source(ESIL *esil, const char *path);
+R_API void *r_anal_esil_get_source(ESIL *esil, ut32 src_id);
+R_API bool r_anal_esil_claim_source(ESIL *esil, ut32 src_id);
+R_API void r_anal_esil_release_source(ESIL *esil, ut32 src_id);
+R_API void r_anal_esil_sources_fini(ESIL *esil);
+
+typedef bool (*RAnalEsilInterruptCB)(ESIL *esil, ut32 interrupt, void *user);
+
+typedef struct r_anal_esil_interrupt_handler_t {
+	const ut32 num;
+	const char* name;
+	void *(*init)(ESIL *esil);
+	RAnalEsilInterruptCB cb;
+	void (*fini)(void *user);
+} RAnalEsilInterruptHandler;
+
+typedef struct r_anal_esil_interrupt_t {
+	RAnalEsilInterruptHandler *handler;
+	void *user;
+	ut32 src_id;
+} RAnalEsilInterrupt;
+
 typedef struct r_anal_esil_session_t {
 	ut64 key;
 	ut64 addr;
@@ -1052,7 +1087,7 @@ typedef struct r_anal_esil_t {
 	ut64 addrmask;
 	int stacksize;
 	int stackptr;
-	int skip;
+	ut32 skip;
 	int nowrite;
 	int iotrap;
 	int exectrap;
@@ -1076,7 +1111,8 @@ typedef struct r_anal_esil_t {
 	ut8 lastsz;	//in bits //used for signature-flag
 	/* native ops and custom ops */
 	Sdb *ops;
-	Sdb *interrupts;
+	RIDStorage *sources;
+	SdbMini *interrupts;
 	/* deep esil parsing fills this */
 	Sdb *stats;
 	Sdb *db_trace;
@@ -1091,7 +1127,7 @@ typedef struct r_anal_esil_t {
 	char *mdev_range; // string containing the r_str_range to match for read/write accesses
 	bool (*cmd)(ESIL *esil, const char *name, ut64 a0, ut64 a1);
 	void *user;
-	int stack_fd;
+	int stack_fd;	// ahem, let's not do this
 	RList *sessions; // <RAnalEsilSession*>
 } RAnalEsil;
 
@@ -1124,7 +1160,6 @@ typedef bool (*RAnalIsValidOffsetCB)(RAnal *anal, ut64 offset, int hasperm);
 
 typedef int (*RAnalEsilCB)(RAnalEsil *esil);
 typedef int (*RAnalEsilLoopCB)(RAnalEsil *esil, RAnalOp *op);
-typedef int (*RAnalEsilInterruptCB)(RAnalEsil *esil, int interrupt);
 typedef int (*RAnalEsilTrapCB)(RAnalEsil *esil, int trap_type, int trap_code);
 
 typedef struct r_anal_plugin_t {
@@ -1207,7 +1242,6 @@ typedef struct r_anal_plugin_t {
 
 	RAnalEsilCB esil_init; // initialize esil-related stuff
 	RAnalEsilLoopCB esil_post_loop;	//cycle-counting, firing interrupts, ...
-	RAnalEsilInterruptCB esil_intr; // interrupts
 	RAnalEsilTrapCB esil_trap; // traps / exceptions
 	RAnalEsilCB esil_fini; // deinitialize
 } RAnalPlugin;
@@ -1227,37 +1261,20 @@ R_API RAnalFunction *r_listrange_find_in_range(RListRange* s, ut64 addr);
 R_API RAnalFunction *r_listrange_find_root(RListRange* s, ut64 addr);
 /* --------- */ /* REFACTOR */ /* ---------- */
 /* type.c */
-R_API int r_anal_type_get_size (RAnal *anal, const char *type);
-R_API RList* r_anal_type_get_by_offset(RAnal *anal, ut64 offset);
 R_API RAnalType *r_anal_type_new(void);
 R_API void r_anal_type_add(RAnal *l, RAnalType *t);
-R_API void r_anal_type_del(RAnal *l, const char *name);
-R_API RList *r_anal_type_list_new(void);
 R_API RAnalType *r_anal_type_find(RAnal *a, const char* name);
 R_API void r_anal_type_list(RAnal *a, short category, short enabled);
 R_API RAnalType *r_anal_str_to_type(RAnal *a, const char* s);
-R_API char *r_anal_type_to_str(RAnal *a, const char *name);
+R_API bool r_anal_op_nonlinear(int t);
+R_API bool r_anal_op_ismemref(int t);
 R_API const char *r_anal_optype_to_string(int t);
 R_API const char *r_anal_op_family_to_string (int n);
 R_API int r_anal_op_family_from_string(const char *f);
 R_API int r_anal_op_hint(RAnalOp *op, RAnalHint *hint);
 R_API RAnalType *r_anal_type_free(RAnalType *t);
 R_API RAnalType *r_anal_type_loadfile(RAnal *a, const char *path);
-R_API void r_anal_type_define (RAnal *anal, const char *key, const char *value);
-R_API void r_anal_type_header (RAnal *anal, const char *hdr);
 
-R_API int r_anal_type_link (RAnal *anal, const char *val, ut64 addr);
-R_API int r_anal_type_unlink(RAnal *anal, ut64 addr);
-R_API int r_anal_type_link_offset (RAnal *anal, const char *val, ut64 addr);
-R_API char *r_anal_type_format (RAnal *anal, const char *t);
-R_API int r_anal_type_set(RAnal *anal, ut64 at, const char *field, ut64 val);
-R_API int r_anal_type_func_exist(RAnal *anal, const char *func_name);
-R_API const char *r_anal_type_func_cc(RAnal *anal, const char *func_name);
- R_API const char *r_anal_type_func_ret(RAnal *anal, const char *func_name);
-R_API int r_anal_type_func_args_count(RAnal *anal, const char *func_name);
-R_API char *r_anal_type_func_args_type(RAnal *anal, const char *func_name, int i);
-R_API char *r_anal_type_func_args_name(RAnal *anal, const char *func_name, int i);
-R_API char *r_anal_type_func_guess(RAnal *anal, char *func_name);
 /* anal.c */
 R_API RAnal *r_anal_new(void);
 R_API int r_anal_purge (RAnal *anal);
@@ -1265,7 +1282,6 @@ R_API RAnal *r_anal_free(RAnal *r);
 R_API void r_anal_set_user_ptr(RAnal *anal, void *user);
 R_API void r_anal_plugin_free (RAnalPlugin *p);
 R_API int r_anal_add(RAnal *anal, RAnalPlugin *foo);
-R_API void r_anal_list(RAnal *anal);
 R_API int r_anal_archinfo(RAnal *anal, int query);
 R_API bool r_anal_use(RAnal *anal, const char *name);
 R_API bool r_anal_set_reg_profile(RAnal *anal);
@@ -1302,6 +1318,7 @@ R_API const char *r_anal_stackop_tostring (int s);
 R_API RAnalOp *r_anal_op_new(void);
 R_API void r_anal_op_free(void *op);
 R_API bool r_anal_op_fini(RAnalOp *op);
+R_API RAnalVar *get_link_var(RAnal *anal, ut64 faddr, RAnalVar *var);
 R_API bool r_anal_op_is_eob (RAnalOp *op);
 R_API RList *r_anal_op_list_new(void);
 R_API int r_anal_op(RAnal *anal, RAnalOp *op, ut64 addr,
@@ -1332,8 +1349,16 @@ R_API void r_anal_esil_stack_free (RAnalEsil *esil);
 R_API int r_anal_esil_get_parm_type (RAnalEsil *esil, const char *str);
 R_API int r_anal_esil_get_parm (RAnalEsil *esil, const char *str, ut64 *num);
 R_API int r_anal_esil_condition (RAnalEsil *esil, const char *str);
-R_API int r_anal_esil_set_interrupt (RAnalEsil *esil, int interrupt, RAnalEsilInterruptCB interruptcb);
-R_API int r_anal_esil_fire_interrupt (RAnalEsil *esil, int interrupt);
+
+// esil_interrupt.c
+R_API void r_anal_esil_interrupts_init (RAnalEsil *esil);
+R_API RAnalEsilInterrupt *r_anal_esil_interrupt_new (RAnalEsil *esil, ut32 src_id,  RAnalEsilInterruptHandler *ih);
+R_API void r_anal_esil_interrupt_free (RAnalEsil *esil, RAnalEsilInterrupt *intr);
+R_API bool r_anal_esil_set_interrupt (RAnalEsil *esil, RAnalEsilInterrupt *intr);
+R_API int r_anal_esil_fire_interrupt (RAnalEsil *esil, ut32 intr_num);
+R_API bool r_anal_esil_load_interrupts (RAnalEsil *esil, RAnalEsilInterruptHandler **handlers, ut32 src_id);
+R_API bool r_anal_esil_load_interrupts_from_lib (RAnalEsil *esil, const char *path);
+R_API void r_anal_esil_interrupts_fini (RAnalEsil *esil);
 
 R_API void r_anal_esil_mem_ro(RAnalEsil *esil, int mem_readonly);
 R_API void r_anal_esil_stats(RAnalEsil *esil, int enable);
@@ -1367,7 +1392,6 @@ R_API RAnalFunction *r_anal_fcn_find_name(RAnal *anal, const char *name);
 R_API RList *r_anal_fcn_list_new(void);
 R_API int r_anal_fcn_insert(RAnal *anal, RAnalFunction *fcn);
 R_API void r_anal_fcn_free(void *fcn);
-R_API void r_anal_fcn_fill_args (RAnal *anal, RAnalFunction *fcn, RAnalOp *op);
 R_API int r_anal_fcn(RAnal *anal, RAnalFunction *fcn, ut64 addr,
 		ut8 *buf, ut64 len, int reftype);
 R_API int r_anal_fcn_add(RAnal *anal, ut64 addr, ut64 size,
@@ -1380,6 +1404,7 @@ R_API int r_anal_fcn_add_bb(RAnal *anal, RAnalFunction *fcn,
 R_API bool r_anal_check_fcn(RAnal *anal, ut8 *buf, ut16 bufsz, ut64 addr, ut64 low, ut64 high);
 R_API void r_anal_fcn_update_tinyrange_bbs(RAnalFunction *fcn);
 
+
 /* locals */
 #if 0
 R_API int r_anal_fcn_local_add(RAnal *anal, RAnalFunction *fcn, int index, const char *name, const char *type);
@@ -1397,8 +1422,7 @@ R_ANAL_FCN_VARKIND_LOCAL, z)
 
 
 R_API int r_anal_fcn_var_del_bydelta (RAnal *a, ut64 fna, const char kind, int scope, ut32 delta);
-R_API int r_anal_fcn_var_del_byindex (RAnal *a, ut64 fna, const char kind,
-		int scope, ut32 idx);
+R_API int r_anal_fcn_var_del_byindex (RAnal *a, ut64 fna, const char kind, int scope, ut32 idx);
 /* args */
 R_API int r_anal_var_count(RAnal *a, RAnalFunction *fcn, int kind, int type);
 
@@ -1414,76 +1438,67 @@ R_API int r_anal_fcn_split_bb(RAnal *anal, RAnalFunction *fcn, RAnalBlock *bb, u
 R_API int r_anal_fcn_bb_overlaps(RAnalFunction *fcn, RAnalBlock *bb);
 R_API RAnalVar *r_anal_fcn_get_var(RAnalFunction *fs, int num, int dir);
 R_API void r_anal_fcn_fit_overlaps (RAnal *anal, RAnalFunction *fcn);
-R_API void r_anal_trim_jmprefs(RAnalFunction *fcn);
+R_API void r_anal_trim_jmprefs(RAnal *anal, RAnalFunction *fcn);
 R_API RAnalFunction *r_anal_fcn_next(RAnal *anal, ut64 addr);
 R_API char *r_anal_fcn_to_string(RAnal *a, RAnalFunction* fs);
 R_API int r_anal_str_to_fcn(RAnal *a, RAnalFunction *f, const char *_str);
 R_API int r_anal_fcn_count (RAnal *a, ut64 from, ut64 to);
-R_API RAnalBlock *r_anal_fcn_bbget(RAnalFunction *fcn, ut64 addr);
+R_API RAnalBlock *r_anal_fcn_bbget_in(RAnalFunction *fcn, ut64 addr);
+R_API RAnalBlock *r_anal_fcn_bbget_at(RAnalFunction *fcn, ut64 addr);
+R_API bool r_anal_fcn_contains(RAnalFunction *fcn, ut64 addr);
 R_API bool r_anal_fcn_bbadd(RAnalFunction *fcn, RAnalBlock *bb);
 R_API int r_anal_fcn_resize (const RAnal *anal, RAnalFunction *fcn, int newsize);
 
-#if 0
-#define r_anal_fcn_get_refs(x) x->refs
-#define r_anal_fcn_get_xrefs(x) x->xrefs
-#define r_anal_fcn_get_vars(x) x->vars
-#define r_anal_fcn_get_bbs(x) x->bbs
-#else
+typedef bool (* RAnalRefCmp)(RAnalRef *ref, void *data);
+R_API RList *r_anal_ref_list_new(void);
 R_API int r_anal_xrefs_count(RAnal *anal);
-R_API const char *r_anal_xrefs_type_tostring (char type);
-R_API RList *r_anal_xrefs_get (RAnal *anal, ut64 to);
-R_API RList *r_anal_refs_get (RAnal *anal, ut64 to);
-R_API RList *r_anal_xrefs_get_from (RAnal *anal, ut64 from);
+R_API const char *r_anal_xrefs_type_tostring(RAnalRefType type);
+R_API RAnalRefType r_anal_xrefs_type(char ch);
+R_API RList *r_anal_xrefs_get(RAnal *anal, ut64 to);
+R_API RList *r_anal_refs_get(RAnal *anal, ut64 to);
+R_API RList *r_anal_xrefs_get_from(RAnal *anal, ut64 from);
 R_API void r_anal_xrefs_list(RAnal *anal, int rad);
 R_API RList *r_anal_fcn_get_refs(RAnal *anal, RAnalFunction *fcn);
 R_API RList *r_anal_fcn_get_xrefs(RAnal *anal, RAnalFunction *fcn);
-R_API int r_anal_xrefs_from (RAnal *anal, RList *list, const char *kind, const RAnalRefType type, ut64 addr);
-R_API int r_anal_xrefs_set (RAnal *anal, const RAnalRefType type, ut64 from, ut64 to);
-R_API int r_anal_xrefs_deln (RAnal *anal, const RAnalRefType type, ut64 from, ut64 to);
-R_API bool r_anal_xrefs_save(RAnal *anal, const char *prjfile);
+R_API int r_anal_xrefs_from(RAnal *anal, RList *list, const char *kind, const RAnalRefType type, ut64 addr);
+R_API int r_anal_xrefs_set(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
+R_API int r_anal_xrefs_deln(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
+R_API int r_anal_xref_del(RAnal *anal, ut64 at, ut64 addr);
+
 R_API RList* r_anal_fcn_get_vars (RAnalFunction *anal);
 R_API RList* r_anal_fcn_get_bbs (RAnalFunction *anal);
 R_API RList* r_anal_get_fcns (RAnal *anal);
-#endif
-
-/* ref.c */
-R_API RAnalRef *r_anal_ref_new(void);
-R_API RList *r_anal_ref_list_new(void);
-R_API void r_anal_ref_free(void *ref);
-R_API const char *r_anal_ref_to_string(int type);
-R_API int r_anal_ref_add(RAnal *anal, ut64 addr, ut64 at, int type);
-R_API int r_anal_ref_del(RAnal *anal, ut64 at, ut64 addr);
-R_API RList *r_anal_xref_get(RAnal *anal, ut64 addr);
-R_API RList *r_anal_ref_get(RAnal *anal, ut64 addr);
 
 /* var.c */
 R_API void r_anal_var_access_clear (RAnal *a, ut64 var_addr, int scope, int index);
 R_API int r_anal_var_access (RAnal *a, ut64 var_addr, char kind, int scope, int index, int xs_type, ut64 xs_addr);
-
 R_API RAnalVar *r_anal_var_new(void);
-R_API int r_anal_var_rename (RAnal *a, ut64 var_addr, int scope, char kind, const char *old_name, const char *new_name);
-R_API int r_anal_var_retype (RAnal *a, ut64 addr, int scope, int delta, char kind, const char *type, int size, const char *name);
+R_API int r_anal_var_rename (RAnal *a, ut64 var_addr, int scope, char kind,
+		const char *old_name, const char *new_name, bool verbose);
+R_API int r_anal_var_retype (RAnal *a, ut64 addr, int scope, int delta, char kind,
+		const char *type, int size, bool isarg, const char *name);
 R_API RAnalVarAccess *r_anal_var_access_new(void);
 R_API RList *r_anal_var_list_new(void);
 R_API RList *r_anal_var_access_list_new(void);
 R_API void r_anal_var_free(RAnalVar *var);
 R_API void r_anal_var_access_free(void *access);
-//R_API int r_anal_var_add(RAnal *anal, RAnalFunction *fcn, ut64 from, int delta, int scope,
-//		RAnalType *type, const char *name, int set);
 R_API int r_anal_var_delete_all (RAnal *a, ut64 addr, const char kind);
 R_API int r_anal_var_delete (RAnal *a, ut64 var_addr, const char kind, int scope, int delta);
 R_API bool r_anal_var_delete_byname (RAnal *a, RAnalFunction *fcn, int type, const char *name);
-R_API bool r_anal_var_add (RAnal *a, ut64 addr, int scope, int delta, char kind, const char *type, int size, const char *name);
+R_API bool r_anal_var_add (RAnal *a, ut64 addr, int scope, int delta, char kind,
+		const char *type, int size, bool isarg, const char *name);
 R_API int r_anal_var_del(RAnal *anal, RAnalFunction *fcn, int delta, int scope);
+R_API ut64 r_anal_var_addr(RAnal *a, RAnalFunction *fcn, const char *name);
 R_API RAnalVar *r_anal_var_get (RAnal *a, ut64 addr, char kind, int scope, int index);
 R_API const char *r_anal_var_scope_to_str(RAnal *anal, int scope);
 R_API int r_anal_var_access_add(RAnal *anal, RAnalVar *var, ut64 from, int set);
 R_API int r_anal_var_access_del(RAnal *anal, RAnalVar *var, ut64 from);
+R_API void extract_vars(RAnal *anal, RAnalFunction *fcn, RAnalOp *op);
+R_API void extract_rarg(RAnal *anal, RAnalOp *op, RAnalFunction *fcn, int *reg_set, int *count);
 R_API RAnalVarAccess *r_anal_var_access_get(RAnal *anal, RAnalVar *var, ut64 from);
-R_API RAnalVar *r_anal_var_get_byname (RAnal *anal, RAnalFunction *fcn, const char* name);
+R_API RAnalVar *r_anal_var_get_byname (RAnal *anal, ut64 addr, const char* name);
 
 /* project */
-R_API bool r_anal_project_save(RAnal *anal, const char *prjfile);
 R_API bool r_anal_xrefs_init (RAnal *anal);
 
 #define R_ANAL_THRESHOLDFCN 0.7F
@@ -1534,8 +1549,10 @@ R_API RList *r_anal_var_list_dynamic(RAnal *anal, RAnalFunction *fcn, int kind);
 // calling conventions API
 R_API int r_anal_cc_exist (RAnal *anal, const char *convention);
 R_API const char *r_anal_cc_arg(RAnal *anal, const char *convention, int n);
+R_API int r_anal_cc_max_arg(RAnal *anal, const char *cc);
 R_API const char *r_anal_cc_ret(RAnal *anal, const char *convention);
 R_API const char *r_anal_cc_default(RAnal *anal);
+R_API const char *r_anal_cc_func(RAnal *anal, const char *func_name);
 R_API const char *r_anal_cc_to_constant(RAnal *anal, char *convention);
 R_API bool r_anal_noreturn_at(RAnal *anal, ut64 addr);
 
@@ -1571,6 +1588,7 @@ R_API int r_meta_var_comment_del(RAnal *a, int type, ut64 idx, ut64 addr);
 R_API int r_meta_add(RAnal *m, int type, ut64 from, ut64 size, const char *str);
 R_API int r_meta_add_with_subtype(RAnal *m, int type, int subtype, ut64 from, ut64 size, const char *str);
 R_API RAnalMetaItem *r_meta_find(RAnal *m, ut64 off, int type, int where);
+R_API RAnalMetaItem *r_meta_find_in(RAnal *m, ut64 off, int type, int where);
 R_API int r_meta_cleanup(RAnal *m, ut64 from, ut64 to);
 R_API const char *r_meta_type_to_string(int type);
 R_API RList *r_meta_enumerate(RAnal *a, int type);
@@ -1582,9 +1600,6 @@ R_API RAnalMetaItem *r_meta_item_new(int type);
 R_API bool r_meta_deserialize_val(RAnalMetaItem *it, int type, ut64 from, const char *v);
 R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full);
 
-R_API int r_anal_fcn_xref_add (RAnal *anal, RAnalFunction *fcn, ut64 at, ut64 addr, int type);
-R_API int r_anal_fcn_xref_del (RAnal *anal, RAnalFunction *fcn, ut64 at, ut64 addr, int type);
-
 /* hints */
 
 R_API void r_anal_build_range_on_hints (RAnal *a);
@@ -1608,6 +1623,7 @@ R_API void r_anal_hint_set_size (RAnal *a, ut64 addr, int length);
 R_API void r_anal_hint_set_opcode (RAnal *a, ut64 addr, const char *str);
 R_API void r_anal_hint_set_esil (RAnal *a, ut64 addr, const char *str);
 R_API void r_anal_hint_set_pointer (RAnal *a, ut64 addr, ut64 jump);
+R_API void r_anal_hint_set_ret(RAnal *a, ut64 addr, ut64 val);
 R_API void r_anal_hint_set_high(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_high(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_size(RAnal *a, ut64 addr);
@@ -1617,6 +1633,7 @@ R_API void r_anal_hint_unset_opcode(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_arch(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_syntax(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_pointer(RAnal *a, ut64 addr);
+R_API void r_anal_hint_unset_ret(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_offset(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_jump(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_fail(RAnal *a, ut64 addr);
@@ -1683,8 +1700,7 @@ typedef struct {
 
 typedef struct vtable_info_t {
 	ut64 saddr; //starting address
-	int method_count;
-	RList* methods;
+	RVector methods;
 } RVTableInfo;
 
 typedef struct vtable_method_info_t {
@@ -1692,82 +1708,86 @@ typedef struct vtable_method_info_t {
 	ut64 vtable_offset;  // offset inside the vtable
 } RVTableMethodInfo;
 
-R_API void r_anal_vtable_info_fini(RVTableInfo *vtable);
+R_API void r_anal_vtable_info_free(RVTableInfo *vtable);
 R_API ut64 r_anal_vtable_info_get_size(RVTableContext *context, RVTableInfo *vtable);
 R_API bool r_anal_vtable_begin(RAnal *anal, RVTableContext *context);
+R_API RVTableInfo *r_anal_vtable_parse_at(RVTableContext *context, ut64 addr);
 R_API RList *r_anal_vtable_search(RVTableContext *context);
-R_API RList *r_anal_vtable_get_methods(RVTableContext *context, RVTableInfo *table);
 R_API void r_anal_list_vtables(RAnal *anal, int rad);
 
 /* rtti */
+R_API char *r_anal_rtti_msvc_demangle_class_name(const char *name);
 R_API void r_anal_rtti_msvc_print_complete_object_locator(RVTableContext *context, ut64 addr, int mode);
 R_API void r_anal_rtti_msvc_print_type_descriptor(RVTableContext *context, ut64 addr, int mode);
 R_API void r_anal_rtti_msvc_print_class_hierarchy_descriptor(RVTableContext *context, ut64 addr, int mode);
 R_API void r_anal_rtti_msvc_print_base_class_descriptor(RVTableContext *context, ut64 addr, int mode);
-R_API void r_anal_rtti_msvc_print_at_vtable(RVTableContext *context, ut64 addr, int mode);
+R_API bool r_anal_rtti_msvc_print_at_vtable(RVTableContext *context, ut64 addr, int mode, bool strict);
 
 R_API void r_anal_rtti_itanium_print_class_type_info(RVTableContext *context, ut64 addr, int mode);
 R_API void r_anal_rtti_itanium_print_si_class_type_info(RVTableContext *context, ut64 addr, int mode);
 R_API void r_anal_rtti_itanium_print_vmi_class_type_info(RVTableContext *context, ut64 addr, int mode);
 R_API void r_anal_rtti_itanium_print_at_vtable(RVTableContext *context, ut64 addr, int mode);
 
+R_API char *r_anal_rtti_demangle_class_name(RAnal *anal, const char *name);
 R_API void r_anal_rtti_print_at_vtable(RAnal *anal, ut64 addr, int mode);
 R_API void r_anal_rtti_print_all(RAnal *anal, int mode);
 
+R_API void r_anal_colorize_bb(RAnal *anal, ut64 addr, ut32 color);
+
 /* plugin pointers */
 extern RAnalPlugin r_anal_plugin_null;
-extern RAnalPlugin r_anal_plugin_xap;
+extern RAnalPlugin r_anal_plugin_6502;
+extern RAnalPlugin r_anal_plugin_8051;
+extern RAnalPlugin r_anal_plugin_arc;
+extern RAnalPlugin r_anal_plugin_arm_cs;
+extern RAnalPlugin r_anal_plugin_arm_gnu;
+extern RAnalPlugin r_anal_plugin_avr;
+extern RAnalPlugin r_anal_plugin_bf;
+extern RAnalPlugin r_anal_plugin_chip8;
+extern RAnalPlugin r_anal_plugin_cr16;
+extern RAnalPlugin r_anal_plugin_cris;
+extern RAnalPlugin r_anal_plugin_dalvik;
+extern RAnalPlugin r_anal_plugin_ebc;
+extern RAnalPlugin r_anal_plugin_gb;
+extern RAnalPlugin r_anal_plugin_h8300;
+extern RAnalPlugin r_anal_plugin_hexagon;
+extern RAnalPlugin r_anal_plugin_i4004;
+extern RAnalPlugin r_anal_plugin_i8080;
+extern RAnalPlugin r_anal_plugin_java;
+extern RAnalPlugin r_anal_plugin_m68k_cs;
+extern RAnalPlugin r_anal_plugin_malbolge;
+extern RAnalPlugin r_anal_plugin_mcore;
+extern RAnalPlugin r_anal_plugin_mips_cs;
+extern RAnalPlugin r_anal_plugin_mips_gnu;
+extern RAnalPlugin r_anal_plugin_msp430;
+extern RAnalPlugin r_anal_plugin_nios2;
+extern RAnalPlugin r_anal_plugin_pic;
+extern RAnalPlugin r_anal_plugin_ppc_cs;
+extern RAnalPlugin r_anal_plugin_ppc_gnu;
+extern RAnalPlugin r_anal_plugin_propeller;
+extern RAnalPlugin r_anal_plugin_riscv;
+extern RAnalPlugin r_anal_plugin_rsp;
+extern RAnalPlugin r_anal_plugin_sh;
+extern RAnalPlugin r_anal_plugin_snes;
+extern RAnalPlugin r_anal_plugin_sparc_cs;
+extern RAnalPlugin r_anal_plugin_sparc_gnu;
+extern RAnalPlugin r_anal_plugin_sysz;
 extern RAnalPlugin r_anal_plugin_tms320;
 extern RAnalPlugin r_anal_plugin_tms320c64x;
-extern RAnalPlugin r_anal_plugin_avr;
-extern RAnalPlugin r_anal_plugin_arm_gnu;
-extern RAnalPlugin r_anal_plugin_arm_cs;
+extern RAnalPlugin r_anal_plugin_v810;
+extern RAnalPlugin r_anal_plugin_v850;
+extern RAnalPlugin r_anal_plugin_vax;
+extern RAnalPlugin r_anal_plugin_wasm;
+extern RAnalPlugin r_anal_plugin_ws;
 extern RAnalPlugin r_anal_plugin_x86;
 extern RAnalPlugin r_anal_plugin_x86_cs;
 extern RAnalPlugin r_anal_plugin_x86_im;
-extern RAnalPlugin r_anal_plugin_x86_udis;
 extern RAnalPlugin r_anal_plugin_x86_simple;
-extern RAnalPlugin r_anal_plugin_ppc_gnu;
-extern RAnalPlugin r_anal_plugin_ppc_cs;
-extern RAnalPlugin r_anal_plugin_java;
-extern RAnalPlugin r_anal_plugin_mips_gnu;
-extern RAnalPlugin r_anal_plugin_mips_cs;
-extern RAnalPlugin r_anal_plugin_dalvik;
-extern RAnalPlugin r_anal_plugin_sh;
-extern RAnalPlugin r_anal_plugin_sparc_gnu;
-extern RAnalPlugin r_anal_plugin_bf;
-extern RAnalPlugin r_anal_plugin_m68k;
-extern RAnalPlugin r_anal_plugin_m68k_cs;
-extern RAnalPlugin r_anal_plugin_z80;
-extern RAnalPlugin r_anal_plugin_i8080;
-extern RAnalPlugin r_anal_plugin_8051;
-extern RAnalPlugin r_anal_plugin_arc;
-extern RAnalPlugin r_anal_plugin_ebc;
-extern RAnalPlugin r_anal_plugin_gb;
-extern RAnalPlugin r_anal_plugin_nios2;
-extern RAnalPlugin r_anal_plugin_malbolge;
-extern RAnalPlugin r_anal_plugin_hexagon;
-extern RAnalPlugin r_anal_plugin_wasm;
-extern RAnalPlugin r_anal_plugin_ws;
-extern RAnalPlugin r_anal_plugin_h8300;
-extern RAnalPlugin r_anal_plugin_cr16;
-extern RAnalPlugin r_anal_plugin_v850;
-extern RAnalPlugin r_anal_plugin_sysz;
-extern RAnalPlugin r_anal_plugin_sparc_cs;
+extern RAnalPlugin r_anal_plugin_x86_udis;
+extern RAnalPlugin r_anal_plugin_xap;
 extern RAnalPlugin r_anal_plugin_xcore_cs;
-extern RAnalPlugin r_anal_plugin_propeller;
-extern RAnalPlugin r_anal_plugin_msp430;
-extern RAnalPlugin r_anal_plugin_chip8;
-extern RAnalPlugin r_anal_plugin_cris;
-extern RAnalPlugin r_anal_plugin_v810;
-extern RAnalPlugin r_anal_plugin_6502;
-extern RAnalPlugin r_anal_plugin_snes;
-extern RAnalPlugin r_anal_plugin_riscv;
-extern RAnalPlugin r_anal_plugin_vax;
-extern RAnalPlugin r_anal_plugin_i4004;
 extern RAnalPlugin r_anal_plugin_xtensa;
-extern RAnalPlugin r_anal_plugin_pic;
-extern RAnalPlugin r_anal_plugin_rsp;
+extern RAnalPlugin r_anal_plugin_z80;
 #ifdef __cplusplus
 }
 #endif
