@@ -21,7 +21,6 @@ typedef int (*RConfigCallback)(void *user, void *data);
 
 typedef struct r_config_node_t {
 	char *name;
-	ut32 hash;
 	int flags;
 	char *value;
 	ut64 i_value;
@@ -31,6 +30,7 @@ typedef struct r_config_node_t {
 	RConfigCallback getter;
 	RConfigCallback setter;
 	char *desc;
+	RList *options;
 } RConfigNode;
 
 R_API const char *r_config_node_type(RConfigNode *node);
@@ -43,10 +43,32 @@ typedef struct r_config_t {
 	RNum *num;
 	PrintfCallback cb_printf;
 	RList *nodes;
-	RHashTable *ht;
+	SdbHt *ht;
 } RConfig;
 
+typedef struct r_config_hold_num_t {
+	char *key;
+	ut64 value;
+} RConfigHoldNum;
+
+typedef struct r_config_hold_char_t {
+	char *key;
+	const char *value;
+} RConfigHoldChar;
+
+typedef struct r_config_hold_t { 
+	RConfig *cfg;
+	RList *list_num; //list of RConfigHoldNum to hold numeric values 
+	RList *list_char; //list of RConfigHoldChar to hold char values
+} RConfigHold;
+
 #ifdef R_API
+R_API bool r_config_save_num(RConfigHold *h, ...);
+R_API bool r_config_save_char(RConfigHold *h, ...);
+R_API RConfigHold* r_config_hold_new(RConfig *cfg);
+R_API void r_config_hold_free(RConfigHold *h);
+R_API void r_config_restore(RConfigHold *h);
+
 R_API RConfig *r_config_new(void *user);
 R_API RConfig *r_config_clone (RConfig *cfg);
 R_API int r_config_free(RConfig *cfg);
