@@ -11,21 +11,30 @@
 #include "r_util.h"
 #include "r_vector.h"
 
-#define R_IO_READ	4
-#define R_IO_WRITE	2
-#define R_IO_EXEC	1
-#define R_IO_RW		(R_IO_READ|R_IO_WRITE)
-#define R_IO_RWX	(R_IO_READ|R_IO_WRITE|R_IO_EXEC)
-#define R_IO_PRIV	16
-#define R_IO_SHAR	32
-#define R_IO_CREAT	64
-
 #define R_IO_SEEK_SET	0
 #define R_IO_SEEK_CUR	1
 #define R_IO_SEEK_END	2
 
 #define R_IO_UNDOS 64
 
+#if HAVE_PTRACE
+
+#if __sun
+#include <unistd.h>
+#include <sys/types.h>
+#else
+#include <sys/ptrace.h>
+#endif
+
+#if defined(__GLIBC__) && defined(__linux__)
+typedef enum __ptrace_request r_ptrace_request_t;
+typedef void * r_ptrace_data_t;
+#else
+typedef int r_ptrace_request_t;
+typedef int r_ptrace_data_t;
+#endif
+#endif
+
 #if __cplusplus
 extern "C" {
 #endif
@@ -88,9 +97,12 @@ typedef struct r_io_t {
 	RIOUndo undo;
 	SdbList *plugins;
 	char *runprofile;
+#ifdef USE_PTRACE_WRAP
+	struct ptrace_wrap_instance_t *ptrace_wrap;
+#endif
 	char *args;
 	void *user;
-	void (*cb_printf)(const char *str, ...);
+	PrintfCallback cb_printf;
 	int (*cb_core_cmd)(void *user, const char *str);
 	char* (*cb_core_cmdstr)(void *user, const char *str);
 	void (*cb_core_post_write)(void *user, ut64 maddr, ut8 *orig_bytes, int orig_len);
@@ -98,7 +110,7 @@ typedef struct r_io_t {
 
 typedef struct r_io_desc_t {
 	int fd;
-	int flags;
+	int perm;
 	int obsz;	//optimal blocksize// do we really need this here?
 	char *uri;
 	char *name;
@@ -153,6 +165,7 @@ typedef struct r_io_plugin_t {
 	int (*write)(RIO *io, RIODesc *fd, const ut8 *buf, int count);
 	int (*close)(RIODesc *desc);
 	bool (*is_blockdevice)(RIODesc *desc);
+	bool (*is_chardevice)(RIODesc *desc);
 	int (*getpid)(RIODesc *desc);
 	int (*gettid)(RIODesc *desc);
 	bool (*getbase)(RIODesc *desc, ut64 *base);
@@ -165,7 +178,7 @@ typedef struct r_io_plugin_t {
 
 typedef struct r_io_map_t {
 	int fd;
-	int flags;
+	int perm;
 	ut32 id;
 	RInterval itv;
 	ut64 delta; //this delta means paddr when talking about section
@@ -177,13 +190,14 @@ typedef struct r_io_map_skyline_t {
 	RInterval itv;
 } RIOMapSkyline;
 
+// XXX must be deprecated maps should be enough
 typedef struct r_io_section_t {
 	char *name;
 	ut64 paddr;
 	ut64 size;
 	ut64 vaddr;
 	ut64 vsize;
-	int flags;
+	int perm;
 	ut32 id;
 	ut32 bin_id;
 	int arch;
@@ -212,23 +226,6 @@ typedef struct r_io_desc_cache_t {
 	ut8 cdata[R_IO_DESC_CACHE_SIZE];
 } RIODescCache;
 
-typedef struct r_io_access_log_element_t {
-	ut64 vaddr;
-	ut64 paddr;
-	int buf_idx;
-	int expect_len;
-	int len;
-	int fd;
-	int mapid;
-	int flags;
-} RIOAccessLogElement;
-
-typedef struct r_io_access_log_t {
-	bool allocation_failed;
-	ut8 *buf;
-	RList *log;
-} RIOAccessLog;
-
 struct r_io_bind_t;
 
 typedef bool (*RIODescUse) (RIO *io, int fd);
@@ -238,7 +235,6 @@ typedef RIODesc *(*RIOOpen) (RIO *io, const char *uri, int flags, int mode);
 typedef RIODesc *(*RIOOpenAt) (RIO *io, const  char *uri, int flags, int mode, ut64 at);
 typedef bool (*RIOClose) (RIO *io, int fd);
 typedef bool (*RIOReadAt) (RIO *io, ut64 addr, ut8 *buf, int len);
-typedef RIOAccessLog *(*RIOAlReadAt) (RIO *io, ut64 addr, ut8 *buf, int len);
 typedef bool (*RIOWriteAt) (RIO *io, ut64 addr, const ut8 *buf, int len);
 typedef char *(*RIOSystem) (RIO *io, const char* cmd);
 typedef int (*RIOFdOpen) (RIO *io, const char *uri, int flags, int mode);
@@ -253,14 +249,14 @@ typedef bool (*RIOFdIsDbg) (RIO *io, int fd);
 typedef const char *(*RIOFdGetName) (RIO *io, int fd);
 typedef RList *(*RIOFdGetMap) (RIO *io, int fd);
 typedef bool (*RIOFdRemap) (RIO *io, int fd, ut64 addr);
-typedef void (*RIOAlSort) (RIOAccessLog *log);
-typedef void (*RIOAlFree) (RIOAccessLog *log);
-typedef ut8 *(*RIOAlGetFbufByflags) (RIOAccessLog *log, int flags, ut64 *addr, int *len);
 typedef bool (*RIOIsValidOff) (RIO *io, ut64 addr, int hasperm);
 typedef bool (*RIOAddrIsMapped) (RIO *io, ut64 addr);
 typedef SdbList *(*RIOSectionVgetSecsAt) (RIO *io, ut64 vaddr);
 typedef RIOSection *(*RIOSectionVgetSec) (RIO *io, ut64 vaddr);
 typedef RIOSection *(*RIOSectionAdd) (RIO *io, ut64 addr, ut64 vaddr, ut64 size, ut64 vsize, int rwx, const char *name, ut32 bin_id, int fd);
+#if HAVE_PTRACE
+typedef long (*RIOPtraceFn) (RIO *io, r_ptrace_request_t request, pid_t pid, void *addr, r_ptrace_data_t data);
+#endif
 
 typedef struct r_io_bind_t {
 	int init;
@@ -272,7 +268,6 @@ typedef struct r_io_bind_t {
 	RIOOpenAt open_at;
 	RIOClose close;
 	RIOReadAt read_at;
-	RIOAlReadAt al_read_at;	//needed for esil
 	RIOWriteAt write_at;
 	RIOSystem system;
 	RIOFdOpen fd_open;
@@ -287,14 +282,14 @@ typedef struct r_io_bind_t {
 	RIOFdGetName fd_get_name;
 	RIOFdGetMap fd_get_map;
 	RIOFdRemap fd_remap;
-	RIOAlSort al_sort;	//needed for esil
-	RIOAlFree al_free;	//needed for esil
-	RIOAlGetFbufByflags al_buf_byflags;	//needed for esil
 	RIOIsValidOff is_valid_offset;
 	RIOAddrIsMapped addr_is_mapped;
 	RIOSectionVgetSecsAt sections_vget;
 	RIOSectionVgetSec sect_vget;
 	RIOSectionAdd section_add;
+#if HAVE_PTRACE
+	RIOPtraceFn ptrace;
+#endif
 } RIOBind;
 
 //map.c
@@ -338,12 +333,9 @@ R_API int r_io_close_all (RIO *io);
 R_API int r_io_pread_at (RIO *io, ut64 paddr, ut8 *buf, int len);
 R_API int r_io_pwrite_at (RIO *io, ut64 paddr, const ut8 *buf, int len);
 R_API bool r_io_vread_at_mapped(RIO* io, ut64 vaddr, ut8* buf, int len);
-R_API RIOAccessLog *r_io_al_vread_at (RIO *io, ut64 vaddr, ut8 *buf, int len);
-R_API RIOAccessLog *r_io_al_vwrite_at (RIO *io, ut64 vaddr, const ut8 *buf, int len);
 R_API bool r_io_read_at (RIO *io, ut64 addr, ut8 *buf, int len);
 R_API bool r_io_read_at_mapped(RIO *io, ut64 addr, ut8 *buf, int len);
 R_API int r_io_nread_at (RIO *io, ut64 addr, ut8 *buf, int len);
-R_API RIOAccessLog *r_io_al_read_at (RIO *io, ut64 addr, ut8 *buf, int len);
 R_API void r_io_alprint(RList *ls);
 R_API bool r_io_write_at (RIO *io, ut64 addr, const ut8 *buf, int len);
 R_API bool r_io_read (RIO *io, ut8 *buf, int len);
@@ -402,6 +394,7 @@ R_API int r_io_wundo_set(RIO *io, int n, int set);
 R_API bool r_io_desc_init (RIO *io);
 R_API RIODesc *r_io_desc_new (RIO *io, RIOPlugin *plugin, const char *uri, int flags, int mode, void *data);
 R_API RIODesc *r_io_desc_open (RIO *io, const char *uri, int flags, int mode);
+R_API RIODesc *r_io_desc_open_plugin (RIO *io, RIOPlugin *plugin, const char *uri, int flags, int mode);
 R_API bool r_io_desc_close (RIODesc *desc);
 R_API int r_io_desc_read (RIODesc *desc, ut8 *buf, int count);
 R_API int r_io_desc_write (RIODesc *desc, const ut8 *buf, int count);
@@ -413,6 +406,7 @@ R_API ut64 r_io_desc_seek (RIODesc *desc, ut64 offset, int whence);
 R_API bool r_io_desc_resize (RIODesc *desc, ut64 newsize);
 R_API ut64 r_io_desc_size (RIODesc *desc);
 R_API bool r_io_desc_is_blockdevice (RIODesc *desc);
+R_API bool r_io_desc_is_chardevice (RIODesc *desc);
 R_API bool r_io_desc_exchange (RIO *io, int fd, int fdx);	//this should get 2 descs
 R_API bool r_io_desc_is_dbg (RIODesc *desc);
 R_API int r_io_desc_get_pid (RIODesc *desc);
@@ -485,6 +479,7 @@ R_API ut64 r_io_fd_seek (RIO *io, int fd, ut64 addr, int whence);
 R_API ut64 r_io_fd_size (RIO *io, int fd);
 R_API bool r_io_fd_resize (RIO *io, int fd, ut64 newsize);
 R_API bool r_io_fd_is_blockdevice (RIO *io, int fd);
+R_API bool r_io_fd_is_chardevice (RIO *io, int fd);
 R_API int r_io_fd_read_at (RIO *io, int fd, ut64 addr, ut8 *buf, int len);
 R_API int r_io_fd_write_at (RIO *io, int fd, ut64 addr, const ut8 *buf, int len);
 R_API bool r_io_fd_is_dbg (RIO *io, int fd);
@@ -505,12 +500,11 @@ R_API bool r_io_is_valid_offset (RIO *io, ut64 offset, int hasperm);
 R_API bool r_io_addr_is_mapped(RIO *io, ut64 vaddr);
 R_API bool r_io_read_i (RIO* io, ut64 addr, ut64 *val, int size, bool endian);
 R_API bool r_io_write_i (RIO* io, ut64 addr, ut64 *val, int size, bool endian);
-R_API RIOAccessLog *r_io_accesslog_new (void);
-R_API void r_io_accesslog_free (RIOAccessLog *log);
-R_API void r_io_accesslog_sort (RIOAccessLog *log);
-R_API void r_io_accesslog_sqash_ignore_gaps (RIOAccessLog *log);
-R_API void r_io_accesslog_sqash_byflags (RIOAccessLog *log, int flags);
-R_API ut8 *r_io_accesslog_getf_buf_byflags (RIOAccessLog *log, int flags, ut64 *addr, int *len);
+
+#if HAVE_PTRACE
+R_API long r_io_ptrace(RIO *io, r_ptrace_request_t request, pid_t pid, void *addr, r_ptrace_data_t data);
+R_API pid_t r_io_ptrace_fork(RIO *io, void (*child_callback)(void *), void *child_callback_user);
+#endif
 
 extern RIOPlugin r_io_plugin_procpid;
 extern RIOPlugin r_io_plugin_malloc;
