@@ -3,6 +3,7 @@
 
 #include "r_version.h"
 
+#define R_CHECKS_LEVEL 2
 #define DEBUGGER 1
 #define HAVE_DECL_ADDR_NO_RANDOMIZE 1
 #define HAVE_ARC4RANDOM_UNIFORM 0
@@ -17,6 +18,7 @@
 #define R2_PLUGINS "@PLUGINS@"
 #define R2_EXTRAS "@EXTRAS@"
 #define R2_BINDINGS "@BINDINGS@"
+#define R2_DATDIR_R2 "@DATADIR_R2@"
 #define R2_SDB "@SDB@"
 #define R2_ZIGNS "@ZIGNS@"
 #define R2_THEMES "@THEMES@"
@@ -43,6 +45,7 @@
 #define R2_EXTRAS "lib/radare2-extras/" R2_VERSION
 #define R2_BINDINGS "lib/radare2-bindings/" R2_VERSION
 #endif
+#define R2_DATDIR_R2        R_JOIN_2_PATHS ("share", "radare2")
 #define R2_SDB              R_JOIN_3_PATHS ("share", "radare2", R2_VERSION)
 #define R2_ZIGNS            R_JOIN_4_PATHS ("share", "radare2", R2_VERSION, "zigns")
 #define R2_THEMES           R_JOIN_4_PATHS ("share", "radare2", R2_VERSION, "cons")
@@ -71,17 +74,25 @@
 #define R2_HOME_BINRC       R_JOIN_2_PATHS (R2_HOME_DATADIR, "rc.d")
 #define R2_HOME_BIN         R_JOIN_3_PATHS (R2_HOME_DATADIR, "prefix", "bin")
 
+#define R2_HOME_HISTORY     R_JOIN_2_PATHS (R2_HOME_CACHEDIR, "history")
+
 #define R2_HOME_RC          R_JOIN_2_PATHS (R2_HOME_CONFIGDIR, "radare2rc")
 #define R2_HOME_RC_DIR      R_JOIN_2_PATHS (R2_HOME_CONFIGDIR, "radare2rc.d")
-
-#define R2_HOME_HISTORY     R_JOIN_2_PATHS (R2_HOME_CACHEDIR, "history")
+#define R2_GLOBAL_RC        R_JOIN_2_PATHS (R2_DATDIR_R2, "radare2rc")
 
 #define HAVE_LIB_MAGIC 1
 #define USE_LIB_MAGIC 0
+#define HAVE_LIB_XXHASH 0
+#define USE_LIB_XXHASH 0
+
 #ifndef HAVE_LIB_SSL
 #define HAVE_LIB_SSL 1
 #endif
 
+#define HAVE_LIBUV 0
+
+#define HAVE_PTRACE 1
+#define USE_PTRACE_WRAP 1
 #define HAVE_FORK 1
 
 #define WITH_GPL 1
