@@ -26,6 +26,7 @@
 int gettimeofday (struct timeval* p, void* tz);
 #endif
 #include <sys/time.h>
+#include "r_util/r_assert.h"
 #include "r_util/r_itv.h"
 #include "r_util/r_rbtree.h"
 #include "r_util/r_big.h"
@@ -65,7 +66,7 @@ int gettimeofday (struct timeval* p, void* tz);
 #include "r_util/r_utf8.h"
 #include "r_util/r_utf16.h"
 #include "r_util/r_utf32.h"
-#include "r_util/r_id_storage.h"
+#include "r_util/r_idpool.h"
 #include "r_util/r_asn1.h"
 #include "r_util/r_json.h"
 #include "r_util/r_x509.h"
