@@ -3,6 +3,18 @@
 
 #include <r_list.h>
 
+#if __WINDOWS__ && !__CYGWIN__
+#define R_SYS_DEVNULL "nul"
+#else
+#define R_SYS_DEVNULL "/dev/null"
+#endif
+
+#if __linux__ || (__FreeBSD__ && __FreeBSD_version >= 1101000) || (__NetBSD__ && __NetBSD_Version__ >= 700000000)
+#define HAS_CLOCK_NANOSLEEP 1
+#else
+#define HAS_CLOCK_NANOSLEEP 0
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -33,6 +45,7 @@ R_API void r_sys_perror_str(const char *fun);
 #else
 #define r_sys_mkdir_failed() (errno != EEXIST)
 #endif
+R_API const char *r_sys_prefix(const char *pfx);
 R_API bool r_sys_mkdir(const char *dir);
 R_API bool r_sys_mkdirp(const char *dir);
 R_API int r_sys_sleep(int secs);
@@ -73,6 +86,22 @@ R_API char *r_sys_cmd_strf(const char *cmd, ...);
 R_API void r_sys_backtrace(void);
 R_API bool r_sys_tts(const char *txt, bool bg);
 
+#if __WINDOWS__
+#include <intrin.h>
+#define r_sys_breakpoint() { __debugbreak(); }
+#else
+#if __i386__ || __x86_64__
+#define r_sys_breakpoint() __asm__ volatile ("int3");
+#elif __arm64__ || __aarch64__
+#define r_sys_breakpoint() __asm__ volatile ("brk 0");
+#elif __arm__ || __thumb__
+#define r_sys_breakpoint() __asm__ volatile ("bkpt $0");
+#else
+#warning r_sys_breakpoint not implemented for this platform
+#define r_sys_breakpoint() { char *a = NULL; *a = 0; }
+#endif
+#endif
+
 /* syscmd */
 R_API char *r_syscmd_ls(const char *input);
 R_API char *r_syscmd_cat(const char *file);
